from copy import copy

import humps.main as humps
from .types import error_detail, payload
from rest_framework.exceptions import ErrorDetail


@error_detail.field("error")
def resolve_error_detail_error(parent, info, *args, **kwargs):
    """Cast the error as a string to retrieve the message"""
    return str(parent)


def traverse_errors(fields, node, stack=""):
    # Does the list contain errors or more fields?
    if isinstance(node, list):
        if any([isinstance(i, ErrorDetail) for i in node]):
            stack_key = copy(stack)
            fields.append(dict(name=humps.camelize(stack_key), values=node))
        else:
            for i, item in enumerate(node):
                # if item is an empty dict, stop.
                if item == {}:
                    continue
                stack_key = copy(stack)
                stack_key = f"{stack_key}[{i}]"
                traverse_errors(fields, item, stack_key)

    # If node is a dict, use the keys
    if isinstance(node, dict):
        for name, errors in node.items():
            # Copy key so the ref is lost and the stack becomes unique
            stack_key = copy(stack)
            if bool(stack_key):
                if isinstance(name, int):
                    stack_key = f"{stack_key}[{name}]"
                else:
                    stack_key = f"{stack_key}.{name}"
            else:
                stack_key = name
            traverse_errors(fields, errors, stack=stack_key)


@payload.field("errors")
def resolve_payload_errors(parent, info, *args, **kwargs):
    fields = list()
    traverse_errors(fields, parent["errors"])
    return fields



# flake8: noqa
from .abc import Resolver
from .mixins import (
    ListModelMixin,
    InputMixin,
    CreateModelMixin,
    DetailModelMixin,
    UpdateModelMixin,
    DestroyModelMixin,
)
from .model import GenericModelResolver, ModelResolver, ListModelResolver



from ariadne import InterfaceType, ObjectType, QueryType

waffle_type = ObjectType("Waffle")
waffle_flag_type = ObjectType("WaffleFlag")
waffle_switch_type = ObjectType("WaffleSwitch")
waffle_sample_type = ObjectType("WaffleSample")
waffle_item_interface = InterfaceType("WaffleItem")
query = QueryType()


def resolve_waffle_type(obj, *args):
    return f"Waffle{obj.__class__.__name__}"


waffle_item_interface.set_type_resolver(resolve_waffle_type)



from ariadne import InterfaceType, ObjectType
from django.db import models


page_info = ObjectType("PageInfo")


node = InterfaceType("Node")


def resolve_node_type(obj, *args):
    if isinstance(obj, models.Model):
        gtype = getattr(obj, "graphql_type", None)
        if isinstance(gtype, str):
            return gtype
        return obj.__class__.__name__


node.set_type_resolver(resolve_node_type)



"""
TODO: remove dependency on DRF field
"""
from ariadne import ScalarType

from rest_framework.fields import UUIDField


uuid_field = UUIDField()


uuid = ScalarType(
    "UUID", serializer=uuid_field.to_representation, value_parser=uuid_field.to_internal_value
)



from .backends import GraphQLFilterBackend


class FilterMixin:
    filter_backends = [GraphQLFilterBackend]
    filter_arg = "filter"

    def get_filter_backends(self):
        return self.filter_backends

    def get_filter_arg(self):
        return self.filter_arg

    def filter_queryset(self, queryset):
        """
        Given a queryset, filter it with whichever filter backend is in use.

        You are unlikely to want to override this method, although you may need
        to call it either from a list view, or from a custom `get_object`
        method if you want to apply the configured filtering backend to the
        default queryset.
        """
        for backend in list(self.filter_backends):
            queryset = backend().filter_queryset(self.request, queryset, self)
        return queryset

    def get_filter_data(self):
        """
        Called by the filter backend when determining how to filter
        """
        if self.config.get("reference", False):
            return self.reference_kwargs
        return self.operation_kwargs.get(self.filter_arg, {})

    def get_queryset(self):
        qs = super().get_queryset()
        return self.filter_queryset(qs)
